@file:Suppress("UnstableApiUsage")

import com.github.benmanes.gradle.versions.updates.DependencyUpdatesTask

// Top-level build file where you can add configuration options common to all sub-projects/modules.
buildscript {
    val debug = gradle.startParameter.taskNames.any { it.contains("debug", true) }
    dependencies {
        classpath(libs.gradlePlugin.android)
        classpath(libs.navigation.safe.args.gradle.plugin)
        classpath(libs.gradlePlugin.kotlin)
        classpath(libs.google.services)
        classpath(libs.perf.plugin)
//        classpath(libs.firebase.crashlytics.gradle)
        classpath(libs.booster.gradle.plugin)
        classpath(libs.android.aop.plugin)
        if (!debug) {
            classpath(libs.booster.transform.thread)
            classpath(libs.booster.transform.res.check)
            classpath(libs.booster.transform.finalizer.watchdog.daemon)
            classpath(libs.booster.transform.activity.thread)
            classpath(libs.booster.task.resource.deredundancy)
            classpath(libs.booster.task.compression.processed.res)
            classpath(libs.booster.transform.shared.preferences)
            classpath(libs.com.didiglobal.booster.booster.transform.media.player)
        }
    }
}

plugins {
    alias(libs.plugins.littlerobots)
    alias(libs.plugins.benmanes)
    alias(libs.plugins.ksp) apply false
    alias(libs.plugins.room) apply false
    alias(libs.plugins.trace) apply false
    alias(libs.plugins.sentry) apply false
    alias(libs.plugins.sentry.kotlin) apply false
    alias(libs.plugins.application) apply false
    alias(libs.plugins.kotlin.android) apply false
    alias(libs.plugins.kotlin.serialization) apply false
    alias(libs.plugins.kotlin.parcelize) apply false
    alias(libs.plugins.android.test) apply false
    alias(libs.plugins.android.library) apply false
    alias(libs.plugins.aboutlibraries) apply false
}

tasks.withType<org.jetbrains.kotlin.gradle.internal.KaptWithoutKotlincTask>().configureEach {
    kaptProcessJvmArgs.add("-Xmx3g")
}

tasks.withType<org.jetbrains.kotlin.gradle.tasks.KotlinJvmCompile>().configureEach {
    compilerOptions.languageVersion.set(org.jetbrains.kotlin.gradle.dsl.KotlinVersion.KOTLIN_1_9)
}

tasks.withType<DependencyUpdatesTask> { rejectVersionIf { false } }

tasks.named<DependencyUpdatesTask>("dependencyUpdates").configure {
    checkConstraints = true
    checkBuildEnvironmentConstraints = true
    checkForGradleUpdate = true
    outputFormatter = "json"
    outputDir = "build/dependencyUpdates"
    reportfileName = "report"
}

versionCatalogUpdate { sortByKey.set(true) }

tasks.register("clean", Delete::class) { delete(rootProject.layout.buildDirectory) }

configurations.all { resolutionStrategy.cacheChangingModulesFor(0, TimeUnit.SECONDS) }
